from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class EffectType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    UNKNOWN_EFFECT: _ClassVar[EffectType]
    LOG: _ClassVar[EffectType]
    ADD_PLAN_COMMAND: _ClassVar[EffectType]
    AUTOCOMPLETE_SEARCH_RESULTS: _ClassVar[EffectType]
    ADD_BANNER_ALERT: _ClassVar[EffectType]
    REMOVE_BANNER_ALERT: _ClassVar[EffectType]
    ORIGINATE_ASSESS_COMMAND: _ClassVar[EffectType]
    EDIT_ASSESS_COMMAND: _ClassVar[EffectType]
    DELETE_ASSESS_COMMAND: _ClassVar[EffectType]
    COMMIT_ASSESS_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_ASSESS_COMMAND: _ClassVar[EffectType]
    ORIGINATE_DIAGNOSE_COMMAND: _ClassVar[EffectType]
    EDIT_DIAGNOSE_COMMAND: _ClassVar[EffectType]
    DELETE_DIAGNOSE_COMMAND: _ClassVar[EffectType]
    COMMIT_DIAGNOSE_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_DIAGNOSE_COMMAND: _ClassVar[EffectType]
    ORIGINATE_GOAL_COMMAND: _ClassVar[EffectType]
    EDIT_GOAL_COMMAND: _ClassVar[EffectType]
    DELETE_GOAL_COMMAND: _ClassVar[EffectType]
    COMMIT_GOAL_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_GOAL_COMMAND: _ClassVar[EffectType]
    ORIGINATE_HPI_COMMAND: _ClassVar[EffectType]
    EDIT_HPI_COMMAND: _ClassVar[EffectType]
    DELETE_HPI_COMMAND: _ClassVar[EffectType]
    COMMIT_HPI_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_HPI_COMMAND: _ClassVar[EffectType]
    ORIGINATE_MEDICATION_STATEMENT_COMMAND: _ClassVar[EffectType]
    EDIT_MEDICATION_STATEMENT_COMMAND: _ClassVar[EffectType]
    DELETE_MEDICATION_STATEMENT_COMMAND: _ClassVar[EffectType]
    COMMIT_MEDICATION_STATEMENT_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_MEDICATION_STATEMENT_COMMAND: _ClassVar[EffectType]
    ORIGINATE_PLAN_COMMAND: _ClassVar[EffectType]
    EDIT_PLAN_COMMAND: _ClassVar[EffectType]
    DELETE_PLAN_COMMAND: _ClassVar[EffectType]
    COMMIT_PLAN_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_PLAN_COMMAND: _ClassVar[EffectType]
    ORIGINATE_PRESCRIBE_COMMAND: _ClassVar[EffectType]
    EDIT_PRESCRIBE_COMMAND: _ClassVar[EffectType]
    DELETE_PRESCRIBE_COMMAND: _ClassVar[EffectType]
    COMMIT_PRESCRIBE_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_PRESCRIBE_COMMAND: _ClassVar[EffectType]
    SEND_PRESCRIBE_COMMAND: _ClassVar[EffectType]
    ORIGINATE_QUESTIONNAIRE_COMMAND: _ClassVar[EffectType]
    EDIT_QUESTIONNAIRE_COMMAND: _ClassVar[EffectType]
    DELETE_QUESTIONNAIRE_COMMAND: _ClassVar[EffectType]
    COMMIT_QUESTIONNAIRE_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_QUESTIONNAIRE_COMMAND: _ClassVar[EffectType]
    ORIGINATE_REASON_FOR_VISIT_COMMAND: _ClassVar[EffectType]
    EDIT_REASON_FOR_VISIT_COMMAND: _ClassVar[EffectType]
    DELETE_REASON_FOR_VISIT_COMMAND: _ClassVar[EffectType]
    COMMIT_REASON_FOR_VISIT_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_REASON_FOR_VISIT_COMMAND: _ClassVar[EffectType]
    ORIGINATE_STOP_MEDICATION_COMMAND: _ClassVar[EffectType]
    EDIT_STOP_MEDICATION_COMMAND: _ClassVar[EffectType]
    DELETE_STOP_MEDICATION_COMMAND: _ClassVar[EffectType]
    COMMIT_STOP_MEDICATION_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_STOP_MEDICATION_COMMAND: _ClassVar[EffectType]
    ORIGINATE_UPDATE_GOAL_COMMAND: _ClassVar[EffectType]
    EDIT_UPDATE_GOAL_COMMAND: _ClassVar[EffectType]
    DELETE_UPDATE_GOAL_COMMAND: _ClassVar[EffectType]
    COMMIT_UPDATE_GOAL_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_UPDATE_GOAL_COMMAND: _ClassVar[EffectType]
    ORIGINATE_PERFORM_COMMAND: _ClassVar[EffectType]
    EDIT_PERFORM_COMMAND: _ClassVar[EffectType]
    DELETE_PERFORM_COMMAND: _ClassVar[EffectType]
    COMMIT_PERFORM_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_PERFORM_COMMAND: _ClassVar[EffectType]
    ORIGINATE_INSTRUCT_COMMAND: _ClassVar[EffectType]
    EDIT_INSTRUCT_COMMAND: _ClassVar[EffectType]
    DELETE_INSTRUCT_COMMAND: _ClassVar[EffectType]
    COMMIT_INSTRUCT_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_INSTRUCT_COMMAND: _ClassVar[EffectType]
    ORIGINATE_LAB_ORDER_COMMAND: _ClassVar[EffectType]
    EDIT_LAB_ORDER_COMMAND: _ClassVar[EffectType]
    DELETE_LAB_ORDER_COMMAND: _ClassVar[EffectType]
    COMMIT_LAB_ORDER_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_LAB_ORDER_COMMAND: _ClassVar[EffectType]
    SEND_LAB_ORDER_COMMAND: _ClassVar[EffectType]
    ORIGINATE_FAMILY_HISTORY_COMMAND: _ClassVar[EffectType]
    EDIT_FAMILY_HISTORY_COMMAND: _ClassVar[EffectType]
    DELETE_FAMILY_HISTORY_COMMAND: _ClassVar[EffectType]
    COMMIT_FAMILY_HISTORY_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_FAMILY_HISTORY_COMMAND: _ClassVar[EffectType]
    ORIGINATE_ALLERGY_COMMAND: _ClassVar[EffectType]
    EDIT_ALLERGY_COMMAND: _ClassVar[EffectType]
    DELETE_ALLERGY_COMMAND: _ClassVar[EffectType]
    COMMIT_ALLERGY_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_ALLERGY_COMMAND: _ClassVar[EffectType]
    ORIGINATE_REMOVE_ALLERGY_COMMAND: _ClassVar[EffectType]
    EDIT_REMOVE_ALLERGY_COMMAND: _ClassVar[EffectType]
    DELETE_REMOVE_ALLERGY_COMMAND: _ClassVar[EffectType]
    COMMIT_REMOVE_ALLERGY_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_REMOVE_ALLERGY_COMMAND: _ClassVar[EffectType]
    ORIGINATE_SURGICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    EDIT_SURGICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    DELETE_SURGICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    COMMIT_SURGICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_SURGICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    CREATE_TASK: _ClassVar[EffectType]
    UPDATE_TASK: _ClassVar[EffectType]
    CREATE_TASK_COMMENT: _ClassVar[EffectType]
    UPSERT_TASK_METADATA: _ClassVar[EffectType]
    ORIGINATE_MEDICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    EDIT_MEDICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    DELETE_MEDICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    COMMIT_MEDICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_MEDICAL_HISTORY_COMMAND: _ClassVar[EffectType]
    ADD_OR_UPDATE_PROTOCOL_CARD: _ClassVar[EffectType]
    ORIGINATE_TASK_COMMAND: _ClassVar[EffectType]
    EDIT_TASK_COMMAND: _ClassVar[EffectType]
    DELETE_TASK_COMMAND: _ClassVar[EffectType]
    COMMIT_TASK_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_TASK_COMMAND: _ClassVar[EffectType]
    ORIGINATE_REFILL_COMMAND: _ClassVar[EffectType]
    EDIT_REFILL_COMMAND: _ClassVar[EffectType]
    DELETE_REFILL_COMMAND: _ClassVar[EffectType]
    COMMIT_REFILL_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_REFILL_COMMAND: _ClassVar[EffectType]
    SEND_REFILL_COMMAND: _ClassVar[EffectType]
    ORIGINATE_VITALS_COMMAND: _ClassVar[EffectType]
    EDIT_VITALS_COMMAND: _ClassVar[EffectType]
    DELETE_VITALS_COMMAND: _ClassVar[EffectType]
    COMMIT_VITALS_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_VITALS_COMMAND: _ClassVar[EffectType]
    ORIGINATE_UPDATE_DIAGNOSIS_COMMAND: _ClassVar[EffectType]
    EDIT_UPDATE_DIAGNOSIS_COMMAND: _ClassVar[EffectType]
    DELETE_UPDATE_DIAGNOSIS_COMMAND: _ClassVar[EffectType]
    COMMIT_UPDATE_DIAGNOSIS_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_UPDATE_DIAGNOSIS_COMMAND: _ClassVar[EffectType]
    ORIGINATE_CLOSE_GOAL_COMMAND: _ClassVar[EffectType]
    EDIT_CLOSE_GOAL_COMMAND: _ClassVar[EffectType]
    DELETE_CLOSE_GOAL_COMMAND: _ClassVar[EffectType]
    COMMIT_CLOSE_GOAL_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_CLOSE_GOAL_COMMAND: _ClassVar[EffectType]
    ORIGINATE_REFER_COMMAND: _ClassVar[EffectType]
    EDIT_REFER_COMMAND: _ClassVar[EffectType]
    DELETE_REFER_COMMAND: _ClassVar[EffectType]
    COMMIT_REFER_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_REFER_COMMAND: _ClassVar[EffectType]
    ORIGINATE_CHANGE_MEDICATION_COMMAND: _ClassVar[EffectType]
    EDIT_CHANGE_MEDICATION_COMMAND: _ClassVar[EffectType]
    DELETE_CHANGE_MEDICATION_COMMAND: _ClassVar[EffectType]
    COMMIT_CHANGE_MEDICATION_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_CHANGE_MEDICATION_COMMAND: _ClassVar[EffectType]
    CREATE_QUESTIONNAIRE_RESULT: _ClassVar[EffectType]
    ANNOTATE_PATIENT_CHART_CONDITION_RESULTS: _ClassVar[EffectType]
    ANNOTATE_PATIENT_CHART_DETECTED_ISSUE_RESULTS: _ClassVar[EffectType]
    ANNOTATE_CLAIM_CONDITION_RESULTS: _ClassVar[EffectType]
    SHOW_PATIENT_CHART_SUMMARY_SECTIONS: _ClassVar[EffectType]
    SHOW_PATIENT_PROFILE_SECTIONS: _ClassVar[EffectType]
    PATIENT_PROFILE__ADD_PHARMACY__POST_SEARCH_RESULTS: _ClassVar[EffectType]
    SEND_SURESCRIPTS_ELIGIBILITY_REQUEST: _ClassVar[EffectType]
    SEND_SURESCRIPTS_MEDICATION_HISTORY_REQUEST: _ClassVar[EffectType]
    SEND_SURESCRIPTS_BENEFITS_REQUEST: _ClassVar[EffectType]
    ORIGINATE_EXAM_COMMAND: _ClassVar[EffectType]
    EDIT_EXAM_COMMAND: _ClassVar[EffectType]
    DELETE_EXAM_COMMAND: _ClassVar[EffectType]
    COMMIT_EXAM_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_EXAM_COMMAND: _ClassVar[EffectType]
    ORIGINATE_ROS_COMMAND: _ClassVar[EffectType]
    EDIT_ROS_COMMAND: _ClassVar[EffectType]
    DELETE_ROS_COMMAND: _ClassVar[EffectType]
    COMMIT_ROS_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_ROS_COMMAND: _ClassVar[EffectType]
    ORIGINATE_STRUCTURED_ASSESSMENT_COMMAND: _ClassVar[EffectType]
    EDIT_STRUCTURED_ASSESSMENT_COMMAND: _ClassVar[EffectType]
    DELETE_STRUCTURED_ASSESSMENT_COMMAND: _ClassVar[EffectType]
    COMMIT_STRUCTURED_ASSESSMENT_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_STRUCTURED_ASSESSMENT_COMMAND: _ClassVar[EffectType]
    ORIGINATE_FOLLOW_UP_COMMAND: _ClassVar[EffectType]
    EDIT_FOLLOW_UP_COMMAND: _ClassVar[EffectType]
    DELETE_FOLLOW_UP_COMMAND: _ClassVar[EffectType]
    COMMIT_FOLLOW_UP_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_FOLLOW_UP_COMMAND: _ClassVar[EffectType]
    ORIGINATE_IMAGING_ORDER_COMMAND: _ClassVar[EffectType]
    EDIT_IMAGING_ORDER_COMMAND: _ClassVar[EffectType]
    DELETE_IMAGING_ORDER_COMMAND: _ClassVar[EffectType]
    COMMIT_IMAGING_ORDER_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_IMAGING_ORDER_COMMAND: _ClassVar[EffectType]
    ORIGINATE_RESOLVE_CONDITION_COMMAND: _ClassVar[EffectType]
    EDIT_RESOLVE_CONDITION_COMMAND: _ClassVar[EffectType]
    DELETE_RESOLVE_CONDITION_COMMAND: _ClassVar[EffectType]
    COMMIT_RESOLVE_CONDITION_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_RESOLVE_CONDITION_COMMAND: _ClassVar[EffectType]
    ORIGINATE_ADJUST_PRESCRIPTION_COMMAND: _ClassVar[EffectType]
    EDIT_ADJUST_PRESCRIPTION_COMMAND: _ClassVar[EffectType]
    DELETE_ADJUST_PRESCRIPTION_COMMAND: _ClassVar[EffectType]
    COMMIT_ADJUST_PRESCRIPTION_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_ADJUST_PRESCRIPTION_COMMAND: _ClassVar[EffectType]
    SEND_ADJUST_PRESCRIPTION_COMMAND: _ClassVar[EffectType]
    ORIGINATE_CHART_SECTION_REVIEW_COMMAND: _ClassVar[EffectType]
    ORIGINATE_IMMUNIZATION_STATEMENT_COMMAND: _ClassVar[EffectType]
    EDIT_IMMUNIZATION_STATEMENT_COMMAND: _ClassVar[EffectType]
    DELETE_IMMUNIZATION_STATEMENT_COMMAND: _ClassVar[EffectType]
    COMMIT_IMMUNIZATION_STATEMENT_COMMAND: _ClassVar[EffectType]
    ENTER_IN_ERROR_IMMUNIZATION_STATEMENT_COMMAND: _ClassVar[EffectType]
    COMMAND_AVAILABLE_ACTIONS_RESULTS: _ClassVar[EffectType]
    SHOW_ACTION_BUTTON: _ClassVar[EffectType]
    PATIENT_PORTAL__FORM_RESULT: _ClassVar[EffectType]
    PATIENT_PORTAL__APPOINTMENT_IS_CANCELABLE: _ClassVar[EffectType]
    PATIENT_PORTAL__APPOINTMENT_IS_RESCHEDULABLE: _ClassVar[EffectType]
    PATIENT_PORTAL__APPOINTMENT_SHOW_MEETING_LINK: _ClassVar[EffectType]
    PATIENT_PORTAL__SEND_INVITE: _ClassVar[EffectType]
    PATIENT_PORTAL__SEND_CONTACT_VERIFICATION: _ClassVar[EffectType]
    PATIENT_PORTAL__APPOINTMENTS__SLOTS__POST_SEARCH_RESULTS: _ClassVar[EffectType]
    PATIENT_PORTAL__APPOINTMENTS__FORM_APPOINTMENT_TYPES__PRE_SEARCH_RESULTS: _ClassVar[EffectType]
    PATIENT_PORTAL__APPOINTMENTS__FORM_APPOINTMENT_TYPES__POST_SEARCH_RESULTS: _ClassVar[EffectType]
    PATIENT_PORTAL__APPOINTMENTS__FORM_LOCATIONS__PRE_SEARCH_RESULTS: _ClassVar[EffectType]
    PATIENT_PORTAL__APPOINTMENTS__FORM_LOCATIONS__POST_SEARCH_RESULTS: _ClassVar[EffectType]
    PATIENT_PORTAL__APPOINTMENTS__FORM_PROVIDERS__PRE_SEARCH_RESULTS: _ClassVar[EffectType]
    PATIENT_PORTAL__APPOINTMENTS__FORM_PROVIDERS__POST_SEARCH_RESULTS: _ClassVar[EffectType]
    PATIENT_PORTAL__APPLICATION_CONFIGURATION: _ClassVar[EffectType]
    ADD_APPOINTMENT_LABEL: _ClassVar[EffectType]
    REMOVE_APPOINTMENT_LABEL: _ClassVar[EffectType]
    ADD_BILLING_LINE_ITEM: _ClassVar[EffectType]
    UPDATE_BILLING_LINE_ITEM: _ClassVar[EffectType]
    REMOVE_BILLING_LINE_ITEM: _ClassVar[EffectType]
    SHOW_PATIENT_PORTAL_MENU_ITEMS: _ClassVar[EffectType]
    PORTAL_WIDGET: _ClassVar[EffectType]
    LAUNCH_MODAL: _ClassVar[EffectType]
    SIMPLE_API_RESPONSE: _ClassVar[EffectType]
    SIMPLE_API_WEBSOCKET_ACCEPT: _ClassVar[EffectType]
    SIMPLE_API_WEBSOCKET_DENY: _ClassVar[EffectType]
    SIMPLE_API_WEBSOCKET_BROADCAST: _ClassVar[EffectType]
    UPDATE_USER: _ClassVar[EffectType]
    CREATE_NOTE: _ClassVar[EffectType]
    UPDATE_NOTE: _ClassVar[EffectType]
    FAX_NOTE: _ClassVar[EffectType]
    CREATE_APPOINTMENT: _ClassVar[EffectType]
    UPDATE_APPOINTMENT: _ClassVar[EffectType]
    RESCHEDULE_APPOINTMENT: _ClassVar[EffectType]
    CANCEL_APPOINTMENT: _ClassVar[EffectType]
    CREATE_SCHEDULE_EVENT: _ClassVar[EffectType]
    UPDATE_SCHEDULE_EVENT: _ClassVar[EffectType]
    DELETE_SCHEDULE_EVENT: _ClassVar[EffectType]
    RESCHEDULE_SCHEDULE_EVENT: _ClassVar[EffectType]
    CREATE_PATIENT: _ClassVar[EffectType]
    UPDATE_PATIENT: _ClassVar[EffectType]
    CREATE_MESSAGE: _ClassVar[EffectType]
    SEND_MESSAGE: _ClassVar[EffectType]
    CREATE_AND_SEND_MESSAGE: _ClassVar[EffectType]
    EDIT_MESSAGE: _ClassVar[EffectType]
    PATIENT_METADATA__CREATE_ADDITIONAL_FIELDS: _ClassVar[EffectType]
    UPSERT_PATIENT_METADATA: _ClassVar[EffectType]
    CREATE_PATIENT_EXTERNAL_IDENTIFIER: _ClassVar[EffectType]
    CREATE_PATIENT_PREFERRED_PHARMACIES: _ClassVar[EffectType]
    CREATE_COMPOUND_MEDICATION: _ClassVar[EffectType]
    UPDATE_COMPOUND_MEDICATION: _ClassVar[EffectType]
    PATIENT_CHART__GROUP_ITEMS: _ClassVar[EffectType]
    APPOINTMENT__FORM__PROVIDERS__PRE_SEARCH_RESULTS: _ClassVar[EffectType]
    APPOINTMENT__FORM__LOCATIONS__PRE_SEARCH_RESULTS: _ClassVar[EffectType]
    APPOINTMENT__FORM__VISIT_TYPES__PRE_SEARCH_RESULTS: _ClassVar[EffectType]
    APPOINTMENT__FORM__DURATIONS__PRE_SEARCH_RESULTS: _ClassVar[EffectType]
    APPOINTMENT__FORM__REASON_FOR_VISIT__PRE_SEARCH_RESULTS: _ClassVar[EffectType]
    APPOINTMENT__FORM__PROVIDERS__POST_SEARCH_RESULTS: _ClassVar[EffectType]
    APPOINTMENT__FORM__LOCATIONS__POST_SEARCH_RESULTS: _ClassVar[EffectType]
    APPOINTMENT__FORM__VISIT_TYPES__POST_SEARCH_RESULTS: _ClassVar[EffectType]
    APPOINTMENT__FORM__DURATIONS__POST_SEARCH_RESULTS: _ClassVar[EffectType]
    APPOINTMENT__FORM__REASON_FOR_VISIT__POST_SEARCH_RESULTS: _ClassVar[EffectType]
    APPOINTMENT__FORM__CREATE_ADDITIONAL_FIELDS: _ClassVar[EffectType]
    UPSERT_APPOINTMENT_METADATA: _ClassVar[EffectType]
    APPOINTMENT__SLOTS__POST_SEARCH_RESULTS: _ClassVar[EffectType]
    SHOW_PANEL_SECTIONS: _ClassVar[EffectType]
    REDIRECT_CONTEXT: _ClassVar[EffectType]
    REVENUE__PAYMENT_PROCESSOR__METADATA: _ClassVar[EffectType]
    REVENUE__PAYMENT_PROCESSOR__FORM: _ClassVar[EffectType]
    REVENUE__PAYMENT_PROCESSOR__CREDIT_CARD_TRANSACTION: _ClassVar[EffectType]
    REVENUE__PAYMENT_PROCESSOR__PAYMENT_METHOD: _ClassVar[EffectType]
    REVENUE__PAYMENT_PROCESSOR__PAYMENT_METHOD__ADD_RESPONSE: _ClassVar[EffectType]
    REVENUE__PAYMENT_PROCESSOR__PAYMENT_METHOD__REMOVE_RESPONSE: _ClassVar[EffectType]
    GENERATE_FULL_CHART_PDF: _ClassVar[EffectType]
    CALENDAR__CREATE: _ClassVar[EffectType]
    CALENDAR__EVENT__CREATE: _ClassVar[EffectType]
UNKNOWN_EFFECT: EffectType
LOG: EffectType
ADD_PLAN_COMMAND: EffectType
AUTOCOMPLETE_SEARCH_RESULTS: EffectType
ADD_BANNER_ALERT: EffectType
REMOVE_BANNER_ALERT: EffectType
ORIGINATE_ASSESS_COMMAND: EffectType
EDIT_ASSESS_COMMAND: EffectType
DELETE_ASSESS_COMMAND: EffectType
COMMIT_ASSESS_COMMAND: EffectType
ENTER_IN_ERROR_ASSESS_COMMAND: EffectType
ORIGINATE_DIAGNOSE_COMMAND: EffectType
EDIT_DIAGNOSE_COMMAND: EffectType
DELETE_DIAGNOSE_COMMAND: EffectType
COMMIT_DIAGNOSE_COMMAND: EffectType
ENTER_IN_ERROR_DIAGNOSE_COMMAND: EffectType
ORIGINATE_GOAL_COMMAND: EffectType
EDIT_GOAL_COMMAND: EffectType
DELETE_GOAL_COMMAND: EffectType
COMMIT_GOAL_COMMAND: EffectType
ENTER_IN_ERROR_GOAL_COMMAND: EffectType
ORIGINATE_HPI_COMMAND: EffectType
EDIT_HPI_COMMAND: EffectType
DELETE_HPI_COMMAND: EffectType
COMMIT_HPI_COMMAND: EffectType
ENTER_IN_ERROR_HPI_COMMAND: EffectType
ORIGINATE_MEDICATION_STATEMENT_COMMAND: EffectType
EDIT_MEDICATION_STATEMENT_COMMAND: EffectType
DELETE_MEDICATION_STATEMENT_COMMAND: EffectType
COMMIT_MEDICATION_STATEMENT_COMMAND: EffectType
ENTER_IN_ERROR_MEDICATION_STATEMENT_COMMAND: EffectType
ORIGINATE_PLAN_COMMAND: EffectType
EDIT_PLAN_COMMAND: EffectType
DELETE_PLAN_COMMAND: EffectType
COMMIT_PLAN_COMMAND: EffectType
ENTER_IN_ERROR_PLAN_COMMAND: EffectType
ORIGINATE_PRESCRIBE_COMMAND: EffectType
EDIT_PRESCRIBE_COMMAND: EffectType
DELETE_PRESCRIBE_COMMAND: EffectType
COMMIT_PRESCRIBE_COMMAND: EffectType
ENTER_IN_ERROR_PRESCRIBE_COMMAND: EffectType
SEND_PRESCRIBE_COMMAND: EffectType
ORIGINATE_QUESTIONNAIRE_COMMAND: EffectType
EDIT_QUESTIONNAIRE_COMMAND: EffectType
DELETE_QUESTIONNAIRE_COMMAND: EffectType
COMMIT_QUESTIONNAIRE_COMMAND: EffectType
ENTER_IN_ERROR_QUESTIONNAIRE_COMMAND: EffectType
ORIGINATE_REASON_FOR_VISIT_COMMAND: EffectType
EDIT_REASON_FOR_VISIT_COMMAND: EffectType
DELETE_REASON_FOR_VISIT_COMMAND: EffectType
COMMIT_REASON_FOR_VISIT_COMMAND: EffectType
ENTER_IN_ERROR_REASON_FOR_VISIT_COMMAND: EffectType
ORIGINATE_STOP_MEDICATION_COMMAND: EffectType
EDIT_STOP_MEDICATION_COMMAND: EffectType
DELETE_STOP_MEDICATION_COMMAND: EffectType
COMMIT_STOP_MEDICATION_COMMAND: EffectType
ENTER_IN_ERROR_STOP_MEDICATION_COMMAND: EffectType
ORIGINATE_UPDATE_GOAL_COMMAND: EffectType
EDIT_UPDATE_GOAL_COMMAND: EffectType
DELETE_UPDATE_GOAL_COMMAND: EffectType
COMMIT_UPDATE_GOAL_COMMAND: EffectType
ENTER_IN_ERROR_UPDATE_GOAL_COMMAND: EffectType
ORIGINATE_PERFORM_COMMAND: EffectType
EDIT_PERFORM_COMMAND: EffectType
DELETE_PERFORM_COMMAND: EffectType
COMMIT_PERFORM_COMMAND: EffectType
ENTER_IN_ERROR_PERFORM_COMMAND: EffectType
ORIGINATE_INSTRUCT_COMMAND: EffectType
EDIT_INSTRUCT_COMMAND: EffectType
DELETE_INSTRUCT_COMMAND: EffectType
COMMIT_INSTRUCT_COMMAND: EffectType
ENTER_IN_ERROR_INSTRUCT_COMMAND: EffectType
ORIGINATE_LAB_ORDER_COMMAND: EffectType
EDIT_LAB_ORDER_COMMAND: EffectType
DELETE_LAB_ORDER_COMMAND: EffectType
COMMIT_LAB_ORDER_COMMAND: EffectType
ENTER_IN_ERROR_LAB_ORDER_COMMAND: EffectType
SEND_LAB_ORDER_COMMAND: EffectType
ORIGINATE_FAMILY_HISTORY_COMMAND: EffectType
EDIT_FAMILY_HISTORY_COMMAND: EffectType
DELETE_FAMILY_HISTORY_COMMAND: EffectType
COMMIT_FAMILY_HISTORY_COMMAND: EffectType
ENTER_IN_ERROR_FAMILY_HISTORY_COMMAND: EffectType
ORIGINATE_ALLERGY_COMMAND: EffectType
EDIT_ALLERGY_COMMAND: EffectType
DELETE_ALLERGY_COMMAND: EffectType
COMMIT_ALLERGY_COMMAND: EffectType
ENTER_IN_ERROR_ALLERGY_COMMAND: EffectType
ORIGINATE_REMOVE_ALLERGY_COMMAND: EffectType
EDIT_REMOVE_ALLERGY_COMMAND: EffectType
DELETE_REMOVE_ALLERGY_COMMAND: EffectType
COMMIT_REMOVE_ALLERGY_COMMAND: EffectType
ENTER_IN_ERROR_REMOVE_ALLERGY_COMMAND: EffectType
ORIGINATE_SURGICAL_HISTORY_COMMAND: EffectType
EDIT_SURGICAL_HISTORY_COMMAND: EffectType
DELETE_SURGICAL_HISTORY_COMMAND: EffectType
COMMIT_SURGICAL_HISTORY_COMMAND: EffectType
ENTER_IN_ERROR_SURGICAL_HISTORY_COMMAND: EffectType
CREATE_TASK: EffectType
UPDATE_TASK: EffectType
CREATE_TASK_COMMENT: EffectType
UPSERT_TASK_METADATA: EffectType
ORIGINATE_MEDICAL_HISTORY_COMMAND: EffectType
EDIT_MEDICAL_HISTORY_COMMAND: EffectType
DELETE_MEDICAL_HISTORY_COMMAND: EffectType
COMMIT_MEDICAL_HISTORY_COMMAND: EffectType
ENTER_IN_ERROR_MEDICAL_HISTORY_COMMAND: EffectType
ADD_OR_UPDATE_PROTOCOL_CARD: EffectType
ORIGINATE_TASK_COMMAND: EffectType
EDIT_TASK_COMMAND: EffectType
DELETE_TASK_COMMAND: EffectType
COMMIT_TASK_COMMAND: EffectType
ENTER_IN_ERROR_TASK_COMMAND: EffectType
ORIGINATE_REFILL_COMMAND: EffectType
EDIT_REFILL_COMMAND: EffectType
DELETE_REFILL_COMMAND: EffectType
COMMIT_REFILL_COMMAND: EffectType
ENTER_IN_ERROR_REFILL_COMMAND: EffectType
SEND_REFILL_COMMAND: EffectType
ORIGINATE_VITALS_COMMAND: EffectType
EDIT_VITALS_COMMAND: EffectType
DELETE_VITALS_COMMAND: EffectType
COMMIT_VITALS_COMMAND: EffectType
ENTER_IN_ERROR_VITALS_COMMAND: EffectType
ORIGINATE_UPDATE_DIAGNOSIS_COMMAND: EffectType
EDIT_UPDATE_DIAGNOSIS_COMMAND: EffectType
DELETE_UPDATE_DIAGNOSIS_COMMAND: EffectType
COMMIT_UPDATE_DIAGNOSIS_COMMAND: EffectType
ENTER_IN_ERROR_UPDATE_DIAGNOSIS_COMMAND: EffectType
ORIGINATE_CLOSE_GOAL_COMMAND: EffectType
EDIT_CLOSE_GOAL_COMMAND: EffectType
DELETE_CLOSE_GOAL_COMMAND: EffectType
COMMIT_CLOSE_GOAL_COMMAND: EffectType
ENTER_IN_ERROR_CLOSE_GOAL_COMMAND: EffectType
ORIGINATE_REFER_COMMAND: EffectType
EDIT_REFER_COMMAND: EffectType
DELETE_REFER_COMMAND: EffectType
COMMIT_REFER_COMMAND: EffectType
ENTER_IN_ERROR_REFER_COMMAND: EffectType
ORIGINATE_CHANGE_MEDICATION_COMMAND: EffectType
EDIT_CHANGE_MEDICATION_COMMAND: EffectType
DELETE_CHANGE_MEDICATION_COMMAND: EffectType
COMMIT_CHANGE_MEDICATION_COMMAND: EffectType
ENTER_IN_ERROR_CHANGE_MEDICATION_COMMAND: EffectType
CREATE_QUESTIONNAIRE_RESULT: EffectType
ANNOTATE_PATIENT_CHART_CONDITION_RESULTS: EffectType
ANNOTATE_PATIENT_CHART_DETECTED_ISSUE_RESULTS: EffectType
ANNOTATE_CLAIM_CONDITION_RESULTS: EffectType
SHOW_PATIENT_CHART_SUMMARY_SECTIONS: EffectType
SHOW_PATIENT_PROFILE_SECTIONS: EffectType
PATIENT_PROFILE__ADD_PHARMACY__POST_SEARCH_RESULTS: EffectType
SEND_SURESCRIPTS_ELIGIBILITY_REQUEST: EffectType
SEND_SURESCRIPTS_MEDICATION_HISTORY_REQUEST: EffectType
SEND_SURESCRIPTS_BENEFITS_REQUEST: EffectType
ORIGINATE_EXAM_COMMAND: EffectType
EDIT_EXAM_COMMAND: EffectType
DELETE_EXAM_COMMAND: EffectType
COMMIT_EXAM_COMMAND: EffectType
ENTER_IN_ERROR_EXAM_COMMAND: EffectType
ORIGINATE_ROS_COMMAND: EffectType
EDIT_ROS_COMMAND: EffectType
DELETE_ROS_COMMAND: EffectType
COMMIT_ROS_COMMAND: EffectType
ENTER_IN_ERROR_ROS_COMMAND: EffectType
ORIGINATE_STRUCTURED_ASSESSMENT_COMMAND: EffectType
EDIT_STRUCTURED_ASSESSMENT_COMMAND: EffectType
DELETE_STRUCTURED_ASSESSMENT_COMMAND: EffectType
COMMIT_STRUCTURED_ASSESSMENT_COMMAND: EffectType
ENTER_IN_ERROR_STRUCTURED_ASSESSMENT_COMMAND: EffectType
ORIGINATE_FOLLOW_UP_COMMAND: EffectType
EDIT_FOLLOW_UP_COMMAND: EffectType
DELETE_FOLLOW_UP_COMMAND: EffectType
COMMIT_FOLLOW_UP_COMMAND: EffectType
ENTER_IN_ERROR_FOLLOW_UP_COMMAND: EffectType
ORIGINATE_IMAGING_ORDER_COMMAND: EffectType
EDIT_IMAGING_ORDER_COMMAND: EffectType
DELETE_IMAGING_ORDER_COMMAND: EffectType
COMMIT_IMAGING_ORDER_COMMAND: EffectType
ENTER_IN_ERROR_IMAGING_ORDER_COMMAND: EffectType
ORIGINATE_RESOLVE_CONDITION_COMMAND: EffectType
EDIT_RESOLVE_CONDITION_COMMAND: EffectType
DELETE_RESOLVE_CONDITION_COMMAND: EffectType
COMMIT_RESOLVE_CONDITION_COMMAND: EffectType
ENTER_IN_ERROR_RESOLVE_CONDITION_COMMAND: EffectType
ORIGINATE_ADJUST_PRESCRIPTION_COMMAND: EffectType
EDIT_ADJUST_PRESCRIPTION_COMMAND: EffectType
DELETE_ADJUST_PRESCRIPTION_COMMAND: EffectType
COMMIT_ADJUST_PRESCRIPTION_COMMAND: EffectType
ENTER_IN_ERROR_ADJUST_PRESCRIPTION_COMMAND: EffectType
SEND_ADJUST_PRESCRIPTION_COMMAND: EffectType
ORIGINATE_CHART_SECTION_REVIEW_COMMAND: EffectType
ORIGINATE_IMMUNIZATION_STATEMENT_COMMAND: EffectType
EDIT_IMMUNIZATION_STATEMENT_COMMAND: EffectType
DELETE_IMMUNIZATION_STATEMENT_COMMAND: EffectType
COMMIT_IMMUNIZATION_STATEMENT_COMMAND: EffectType
ENTER_IN_ERROR_IMMUNIZATION_STATEMENT_COMMAND: EffectType
COMMAND_AVAILABLE_ACTIONS_RESULTS: EffectType
SHOW_ACTION_BUTTON: EffectType
PATIENT_PORTAL__FORM_RESULT: EffectType
PATIENT_PORTAL__APPOINTMENT_IS_CANCELABLE: EffectType
PATIENT_PORTAL__APPOINTMENT_IS_RESCHEDULABLE: EffectType
PATIENT_PORTAL__APPOINTMENT_SHOW_MEETING_LINK: EffectType
PATIENT_PORTAL__SEND_INVITE: EffectType
PATIENT_PORTAL__SEND_CONTACT_VERIFICATION: EffectType
PATIENT_PORTAL__APPOINTMENTS__SLOTS__POST_SEARCH_RESULTS: EffectType
PATIENT_PORTAL__APPOINTMENTS__FORM_APPOINTMENT_TYPES__PRE_SEARCH_RESULTS: EffectType
PATIENT_PORTAL__APPOINTMENTS__FORM_APPOINTMENT_TYPES__POST_SEARCH_RESULTS: EffectType
PATIENT_PORTAL__APPOINTMENTS__FORM_LOCATIONS__PRE_SEARCH_RESULTS: EffectType
PATIENT_PORTAL__APPOINTMENTS__FORM_LOCATIONS__POST_SEARCH_RESULTS: EffectType
PATIENT_PORTAL__APPOINTMENTS__FORM_PROVIDERS__PRE_SEARCH_RESULTS: EffectType
PATIENT_PORTAL__APPOINTMENTS__FORM_PROVIDERS__POST_SEARCH_RESULTS: EffectType
PATIENT_PORTAL__APPLICATION_CONFIGURATION: EffectType
ADD_APPOINTMENT_LABEL: EffectType
REMOVE_APPOINTMENT_LABEL: EffectType
ADD_BILLING_LINE_ITEM: EffectType
UPDATE_BILLING_LINE_ITEM: EffectType
REMOVE_BILLING_LINE_ITEM: EffectType
SHOW_PATIENT_PORTAL_MENU_ITEMS: EffectType
PORTAL_WIDGET: EffectType
LAUNCH_MODAL: EffectType
SIMPLE_API_RESPONSE: EffectType
SIMPLE_API_WEBSOCKET_ACCEPT: EffectType
SIMPLE_API_WEBSOCKET_DENY: EffectType
SIMPLE_API_WEBSOCKET_BROADCAST: EffectType
UPDATE_USER: EffectType
CREATE_NOTE: EffectType
UPDATE_NOTE: EffectType
FAX_NOTE: EffectType
CREATE_APPOINTMENT: EffectType
UPDATE_APPOINTMENT: EffectType
RESCHEDULE_APPOINTMENT: EffectType
CANCEL_APPOINTMENT: EffectType
CREATE_SCHEDULE_EVENT: EffectType
UPDATE_SCHEDULE_EVENT: EffectType
DELETE_SCHEDULE_EVENT: EffectType
RESCHEDULE_SCHEDULE_EVENT: EffectType
CREATE_PATIENT: EffectType
UPDATE_PATIENT: EffectType
CREATE_MESSAGE: EffectType
SEND_MESSAGE: EffectType
CREATE_AND_SEND_MESSAGE: EffectType
EDIT_MESSAGE: EffectType
PATIENT_METADATA__CREATE_ADDITIONAL_FIELDS: EffectType
UPSERT_PATIENT_METADATA: EffectType
CREATE_PATIENT_EXTERNAL_IDENTIFIER: EffectType
CREATE_PATIENT_PREFERRED_PHARMACIES: EffectType
CREATE_COMPOUND_MEDICATION: EffectType
UPDATE_COMPOUND_MEDICATION: EffectType
PATIENT_CHART__GROUP_ITEMS: EffectType
APPOINTMENT__FORM__PROVIDERS__PRE_SEARCH_RESULTS: EffectType
APPOINTMENT__FORM__LOCATIONS__PRE_SEARCH_RESULTS: EffectType
APPOINTMENT__FORM__VISIT_TYPES__PRE_SEARCH_RESULTS: EffectType
APPOINTMENT__FORM__DURATIONS__PRE_SEARCH_RESULTS: EffectType
APPOINTMENT__FORM__REASON_FOR_VISIT__PRE_SEARCH_RESULTS: EffectType
APPOINTMENT__FORM__PROVIDERS__POST_SEARCH_RESULTS: EffectType
APPOINTMENT__FORM__LOCATIONS__POST_SEARCH_RESULTS: EffectType
APPOINTMENT__FORM__VISIT_TYPES__POST_SEARCH_RESULTS: EffectType
APPOINTMENT__FORM__DURATIONS__POST_SEARCH_RESULTS: EffectType
APPOINTMENT__FORM__REASON_FOR_VISIT__POST_SEARCH_RESULTS: EffectType
APPOINTMENT__FORM__CREATE_ADDITIONAL_FIELDS: EffectType
UPSERT_APPOINTMENT_METADATA: EffectType
APPOINTMENT__SLOTS__POST_SEARCH_RESULTS: EffectType
SHOW_PANEL_SECTIONS: EffectType
REDIRECT_CONTEXT: EffectType
REVENUE__PAYMENT_PROCESSOR__METADATA: EffectType
REVENUE__PAYMENT_PROCESSOR__FORM: EffectType
REVENUE__PAYMENT_PROCESSOR__CREDIT_CARD_TRANSACTION: EffectType
REVENUE__PAYMENT_PROCESSOR__PAYMENT_METHOD: EffectType
REVENUE__PAYMENT_PROCESSOR__PAYMENT_METHOD__ADD_RESPONSE: EffectType
REVENUE__PAYMENT_PROCESSOR__PAYMENT_METHOD__REMOVE_RESPONSE: EffectType
GENERATE_FULL_CHART_PDF: EffectType
CALENDAR__CREATE: EffectType
CALENDAR__EVENT__CREATE: EffectType

class Effect(_message.Message):
    __slots__ = ("type", "payload", "plugin_name", "classname", "handler_name")
    TYPE_FIELD_NUMBER: _ClassVar[int]
    PAYLOAD_FIELD_NUMBER: _ClassVar[int]
    PLUGIN_NAME_FIELD_NUMBER: _ClassVar[int]
    CLASSNAME_FIELD_NUMBER: _ClassVar[int]
    HANDLER_NAME_FIELD_NUMBER: _ClassVar[int]
    type: EffectType
    payload: str
    plugin_name: str
    classname: str
    handler_name: str
    def __init__(self, type: _Optional[_Union[EffectType, str]] = ..., payload: _Optional[str] = ..., plugin_name: _Optional[str] = ..., classname: _Optional[str] = ..., handler_name: _Optional[str] = ...) -> None: ...



import json
from typing import Annotated, Any
from uuid import UUID

from django.db.models import Count
from pydantic import Field
from pydantic_core import InitErrorDetails

from canvas_sdk.base import TrackableFieldsModel
from canvas_sdk.effects import Effect, EffectType
from canvas_sdk.effects.base import _BaseEffect
from canvas_sdk.effects.note.base import AppointmentABC
from canvas_sdk.v1.data import Appointment as AppointmentDataModel
from canvas_sdk.v1.data import AppointmentLabel, NoteType, Patient
from canvas_sdk.v1.data.note import NoteTypeCategories


class ScheduleEvent(AppointmentABC):
    """
    Effect to create a schedule event.

    Attributes:
        note_type_id (UUID | str | None): The ID of the note type.
        patient_id (str | None): The ID of the patient, if applicable.
        description (str | None): The description of the schedule event, if applicable.
    """

    class Meta:
        effect_type = "SCHEDULE_EVENT"

    note_type_id: UUID | str | None = None
    patient_id: str | None = None
    description: str | None = None

    def _get_error_details(self, method: Any) -> list[InitErrorDetails]:
        """
        Validates the schedule event note type and returns a list of error details if validation fails.

        Args:
            method (Any): The method being validated.

        Returns:
            list[InitErrorDetails]: A list of error details for invalid schedule event note types.
        """
        errors = super()._get_error_details(method)

        # note_type_id is required for create
        if method == "create" and not self.note_type_id:
            errors.append(
                self._create_error_detail(
                    "missing",
                    "Field 'note_type_id' is required to create a schedule event.",
                    None,
                )
            )
            return errors

        if self.note_type_id:
            note_type = (
                NoteType.objects.values("category", "is_patient_required", "allow_custom_title")
                .filter(id=self.note_type_id)
                .first()
            )

            if not note_type:
                errors.append(
                    self._create_error_detail(
                        "value",
                        f"Note type with ID {self.note_type_id} does not exist.",
                        self.note_type_id,
                    )
                )
                return errors

            if note_type["category"] != NoteTypeCategories.SCHEDULE_EVENT:
                errors.append(
                    self._create_error_detail(
                        "value",
                        f"Schedule event note type must be of type: {NoteTypeCategories.SCHEDULE_EVENT}.",
                        self.note_type_id,
                    )
                )

            if note_type["is_patient_required"] and not self.patient_id:
                errors.append(
                    self._create_error_detail(
                        "value",
                        "Patient ID is required for this note type.",
                        self.note_type_id,
                    )
                )

            if not note_type["allow_custom_title"] and self.description:
                errors.append(
                    self._create_error_detail(
                        "value",
                        "Description is not allowed for this note type.",
                        self.note_type_id,
                    )
                )

        # Validate patient exists if provided
        if self.patient_id and not Patient.objects.filter(id=self.patient_id).exists():
            errors.append(
                self._create_error_detail(
                    "value",
                    f"Patient with ID {self.patient_id} does not exist.",
                    self.patient_id,
                )
            )

        return errors

    def delete(self) -> Effect:
        """Send a DELETE effect for the schedule event."""
        self._validate_before_effect("delete")
        return Effect(
            type=f"DELETE_{self.Meta.effect_type}",
            payload=json.dumps(
                {
                    "data": self.values,
                }
            ),
        )


class Appointment(AppointmentABC):
    """
    Effect to create or update an appointment.

    Attributes:
        appointment_note_type_id (UUID | str | None): The ID of the appointment note type.
        meeting_link (str | None): The meeting link for the appointment, if any.
        patient_id (str | None): The ID of the patient.
        labels (conset[str] | None): A set of label names to apply to the appointment.
    """

    class Meta:
        effect_type = "APPOINTMENT"

    appointment_note_type_id: UUID | str | None = None
    meeting_link: str | None = None
    patient_id: str | None = None
    labels: (
        Annotated[
            set[Annotated[str, Field(min_length=1, max_length=50)]],
            Field(min_length=1, max_length=3),
        ]
        | None
    ) = None

    def _get_error_details(self, method: Any) -> list[InitErrorDetails]:
        """
        Validates the appointment note type and returns a list of error details if validation fails.

        Args:
            method (Any): The method being validated.

        Returns:
            list[InitErrorDetails]: A list of error details for invalid appointment note types or missing patient IDs.
        """
        errors = super()._get_error_details(method)

        if method == "create":
            if not self.appointment_note_type_id:
                errors.append(
                    self._create_error_detail(
                        "missing",
                        "Field 'appointment_note_type_id' is required to create an appointment.",
                        None,
                    )
                )
            if not self.patient_id:
                errors.append(
                    self._create_error_detail(
                        "missing",
                        "Field 'patient_id' is required to create an appointment.",
                        None,
                    )
                )

        if method == "update" and self.patient_id:
            errors.append(
                self._create_error_detail(
                    "value",
                    "Field 'patient_id' cannot be updated for an existing appointment.",
                    None,
                )
            )

        if self.patient_id and not Patient.objects.filter(id=self.patient_id).exists():
            errors.append(
                self._create_error_detail(
                    "value",
                    "Patient with ID {self.patient_id} does not exist.",
                    self.patient_id,
                )
            )

        if self.appointment_note_type_id:
            try:
                category, is_scheduleable = NoteType.objects.values_list(
                    "category", "is_scheduleable"
                ).get(id=self.appointment_note_type_id)
                if category != NoteTypeCategories.ENCOUNTER:
                    errors.append(
                        self._create_error_detail(
                            "value",
                            f"Appointment note type must be of type, {NoteTypeCategories.ENCOUNTER} but got: {category}.",
                            self.appointment_note_type_id,
                        )
                    )

                if not is_scheduleable:
                    errors.append(
                        self._create_error_detail(
                            "value",
                            "Appointment note type must be scheduleable.",
                            self.appointment_note_type_id,
                        )
                    )
            except NoteType.DoesNotExist:
                errors.append(
                    self._create_error_detail(
                        "value",
                        f"Note type with ID {self.appointment_note_type_id} does not exist.",
                        self.appointment_note_type_id,
                    )
                )

        if method == "update" and self.labels and self.instance_id:
            existing_count = AppointmentLabel.objects.filter(
                appointment__id=self.instance_id
            ).count()
            if existing_count + len(self.labels) > 3:
                errors.append(
                    self._create_error_detail(
                        "value",
                        f"Limit reached: Only 3 appointment labels allowed. Attempted to add {len(self.labels)} label(s) to appointment with {existing_count} existing label(s).",
                        sorted(self.labels),
                    )
                )

        return errors

    @property
    def values(self) -> dict:
        """
        Returns a dictionary of modified attributes with type-specific transformations.
        """
        values = super().values
        # Convert labels set to list for JSON serialization
        # This is necessary because:
        # 1. The labels field is defined as a conset (constrained set) for validation
        # 2. JSON cannot serialize Python sets directly - it only supports lists, dicts, strings, numbers, booleans, and None
        # 3. When the effect payload is serialized to JSON in the base Effect class, it would fail with:
        #    "TypeError: Object of type set is not JSON serializable"
        # 4. Converting to list maintains the same data while making it JSON-compatible
        # 5. Sort the labels to ensure consistent ordering for tests and API responses
        if self.labels is not None:
            values["labels"] = sorted(self.labels)
        return values

    def cancel(self) -> Effect:
        """Send a CANCEL effect for the appointment."""
        self._validate_before_effect("cancel")
        return Effect(
            type=f"CANCEL_{self.Meta.effect_type}",
            payload=json.dumps(
                {
                    "data": self.values,
                }
            ),
        )


class _AppointmentLabelBase(_BaseEffect, TrackableFieldsModel):
    """
    Base class for appointment label effects.

    Attributes:
        appointment_id (UUID | str): The ID of the appointment.
        labels (conset[str]): A set of label names (1-3 labels allowed).
    """

    appointment_id: str
    labels: Annotated[set[str], Field(min_length=1, max_length=3)]

    @property
    def values(self) -> dict:
        """The effect's values."""
        result = {
            "appointment_id": str(self.appointment_id),
            "labels": sorted(self.labels),
        }
        return result


class AddAppointmentLabel(_AppointmentLabelBase):
    """
    Effect to add one or more labels to an appointment.
    """

    class Meta:
        effect_type = EffectType.ADD_APPOINTMENT_LABEL

    def _get_error_details(self, method: Any) -> list[InitErrorDetails]:
        """Validate that the appointment does not exceed the 3-label limit."""
        errors = super()._get_error_details(method)

        appointment_label_count = (
            AppointmentDataModel.objects.filter(id=self.appointment_id)
            .annotate(label_count=Count("labels"))
            .values_list("label_count", flat=True)
            .first()
        )

        # note that appointment_label_count will be None if the appointment doesn't exist
        # and appointment_label_count will be 0 if the appointment exists but has no labels
        if appointment_label_count is None:
            errors.append(
                self._create_error_detail(
                    "value",
                    f"Appointment {self.appointment_id} does not exist",
                    self.appointment_id,
                )
            )
        elif appointment_label_count + len(self.labels) > 3:
            errors.append(
                self._create_error_detail(
                    "value",
                    f"Limit reached: Only 3 appointment labels allowed. "
                    f"Attempted to add {len(self.labels)} label(s) to appointment with "
                    f"{appointment_label_count} existing label(s).",
                    sorted(self.labels),
                )
            )
        return errors


class RemoveAppointmentLabel(_AppointmentLabelBase):
    """
    Effect to remove one or more labels from an appointment.

    Attributes:
        appointment_id (UUID | str): The ID of the appointment to remove labels from.
        labels (list[str]): A list of label names to remove.
    """

    class Meta:
        effect_type = EffectType.REMOVE_APPOINTMENT_LABEL


__exports__ = (
    "ScheduleEvent",
    "Appointment",
    "AddAppointmentLabel",
    "RemoveAppointmentLabel",
)



from datetime import date
from typing import Self

from pydantic import model_validator
from pydantic_core import InitErrorDetails

from canvas_sdk.commands.base import _BaseCommand as BaseCommand
from canvas_sdk.commands.constants import CodeSystems, Coding


class ImmunizationStatementCommand(BaseCommand):
    """A class for managing an ImmunizationStatement command within a specific note."""

    class Meta:
        key = "immunizationStatement"

    cpt_code: str | Coding | None = None
    cvx_code: str | Coding | None = None
    unstructured: Coding | None = None
    approximate_date: date | None = None
    comments: str | None = None

    def _has_value(self, value: str | Coding | None) -> bool:
        """Check if a value is set."""
        if value is None:
            return False
        if isinstance(value, str):
            return value.strip() != ""
        # For Coding objects (dicts), check if it exists
        return True

    @model_validator(mode="after")
    def check_needed_together_fields(self) -> Self:
        """Check that both 'cpt_code' and 'cvx_code' are set if one is provided."""
        has_cpt_code = self._has_value(self.cpt_code)
        has_cvx_code = self._has_value(self.cvx_code)

        if has_cpt_code ^ has_cvx_code:
            raise ValueError(
                "Both cpt_code and cvx_code must be provided if one is specified and cannot be empty."
            )

        return self

    def _get_error_details(self, method: str) -> list[InitErrorDetails]:
        errors = super()._get_error_details(method)

        if self.unstructured and (self.cpt_code or self.cvx_code):
            message = "Unstructured codes cannot be used with CPT or CVX codes."
            errors.append(self._create_error_detail("value", message, self.cpt_code))

        if isinstance(self.cpt_code, dict) and self.cpt_code["system"] != CodeSystems.CPT:
            message = f"The 'cpt_code.system' field must be '{CodeSystems.CPT}'"
            errors.append(self._create_error_detail("value", message, self.cpt_code))

        if isinstance(self.cvx_code, dict) and self.cvx_code["system"] != CodeSystems.CVX:
            message = f"The 'cvx_code.system' field must be '{CodeSystems.CVX}'"
            errors.append(self._create_error_detail("value", message, self.cvx_code))

        if self.unstructured and self.unstructured["system"] != CodeSystems.UNSTRUCTURED:
            message = f"The 'unstructured.system' field must be '{CodeSystems.UNSTRUCTURED}'"
            errors.append(self._create_error_detail("value", message, self.unstructured))

        if self.comments and len(self.comments) > 255:
            errors.append(
                self._create_error_detail(
                    "comments", "Comments must be 255 characters or less.", self.comments
                )
            )

        return errors


__exports__ = ("ImmunizationStatementCommand",)



from .api import SimpleAPI, SimpleAPIRoute
from .security import (
    APIKeyAuthMixin,
    APIKeyCredentials,
    AuthSchemeMixin,
    BasicAuthMixin,
    BasicCredentials,
    BearerCredentials,
    Credentials,
    PatientSessionAuthMixin,
    SessionCredentials,
    StaffSessionAuthMixin,
)

__all__ = __exports__ = (
    "api",
    "APIKeyAuthMixin",
    "APIKeyCredentials",
    "AuthSchemeMixin",
    "BasicAuthMixin",
    "BasicCredentials",
    "BearerCredentials",
    "Credentials",
    "PatientSessionAuthMixin",
    "SessionCredentials",
    "SimpleAPI",
    "SimpleAPIRoute",
    "StaffSessionAuthMixin",
)
